/* hmmvpex: Examples for hmmvp
 *   Version 0.1
 *   Andrew M. Bradley
 *   ambrad@cs.stanford.edu
 *   CDFM Group, Geophysics, Stanford
 *   https://pangea.stanford.edu/research/CDFM/software
 * hmmvpex is licensed as follows:
 *   Open Source Initiative OSI - Eclipse Public License 1.0
 *   http://www.opensource.org/licenses/eclipse-1.0
*/

#ifndef INCLUDE_HMMVP_HELP_TEXT
#define INCLUDE_HMMVP_HELP_TEXT
/* Help text for programs. I write the text in a plain text file and then run
   this routine:

    function WrapPrintHelp (fn)
      fid = fopen(fn, 'r');
      state = 0;
      while (true)
        ln = fgetl(fid);
        if (~ischar(ln)) break; end
        if (numel(ln) >= 10 && strcmp(ln(1:7), '#define'))
          if (state == 1) fprintf('"\n\n'); end
          fprintf('%s \\\n"', ln);
          state = 1;
        else fprintf('%s\\n\\\n', ln); end
      end
      fprintf('"\n\n');
      fclose(fid);
    end
*/

#define _hmmvp_Header_text_ \
"hmmvp: Software to form and apply Hierarchical Matrices\n\
Version 1.2\n\
A.M. Bradley ambrad@cs.stanford.edu\n\
CDFM Group, Geophysics, Stanford University\n\
\n\
\n\
"

#define _hmmvpbuild_PrintHelp_text_ \
"hmmvpbuild\n\
  This message.\n\
\n\
hmmvpbuild <key-value file>\n\
  Build an H-matrix.\n\
    The key-value file describes the problem. The easiest way to create a\n\
  key-value file is to use matlab/kvf.m. You can create one in C++ by using the\n\
  interface described in util/include/KeyValueFile.hpp and linking against\n\
  libhmmvp.\n\
    The key-value file must contain at least this field:\n\
      command [string]: Right now, there is just one command available:\n\
        compress .\n\
    Type 'hmmvp help <command>' for help on each command.\n\
\n\
hmmvpbuild help <command>\n\
\n\
\n\
"

#define _hmmvpbuild_compress_PrintHelp_text_ \
"compress: Build an H-matrix. The key-value file must contain the following\n\
additional fields:\n\
\n\
  write_hmat_filename [string]: Filename prefix to which to write H-matrix\n\
    data. The extension .hm is appended if it is not already present.\n\
\n\
  eta [real scalar, [3]]: eta controls how point clusters are formed and\n\
    paired. Let c1 and c2 be two point clusters. For clusters c, c1, c2 and\n\
    points p, p1, p2 define:\n\
      centroid(c): Unweighted centroid of the points\n\
      dist(p1, p2): Euclidean distance between p1 and p2\n\
      radius(c): max_{point p in cluster c} dist(p, centroid(c)).\n\
    Then two clusters are considered to be sufficiently distant if\n\
      min(radius(c1), radius(c2)) <\n\
        eta/2 [dist(centroid(c1), centroid(c2)) - (radius(c1) + radius(c2))].\n\
\n\
  tol [real scalar, [1e-6]]: The error tolerance for the H-matrix\n\
    approximation. See next.\n\
\n\
  err_method [['mrem-fro'], 'brem-fro', or 'mrem-abs']: There are two parts to\n\
    each error method:\n\
      1. MREM vs BREM and\n\
      2. (Abs)olute or relative to ||G||_F, the (Fro)benius norm of the matrix.\n\
    If 'abs', then the error bound of the approximation Ga to G has the form\n\
      ||Ga - G||_F <= tol;\n\
    if 'fro', then\n\
      ||Ga - G||_F <= tol ||G||_F.\n\
    Cal the right-hand-side quantity 'err'. Then each has the form\n\
      ||Ga - G||_F <= err.\n\
    MREM and BREM differ in how they achieve this bound by assigning different\n\
    error tolerances to each block Ga_i. brem-fro implements\n\
      ||Ga_i - G_i||_F <= tol ||G_i||_F.\n\
    mrem-(fro/abs) implements\n\
      ||Ga_i - G_i||_F  <= sqrt(m_i n_i) / sqrt(M N) err,\n\
    where G is M x N and G_i is m_i x n_i.\n\
      MREM is the more efficient method of MREM and BREM in the following\n\
    sense. It achieves the greater compression for a given tol. But BREM may be\n\
    preferable when you want something closer to an element-wise, rather than\n\
    norm-wise, relative error. (This issue is somewhat complicated.) I recommend\n\
    you stick with the default unless you have a reason to switch.\n\
\n\
  Bfro [real scalar; optional]: The user's estimate of the Frobenius norm of the\n\
    matrix. If one is not provided but is needed, then an internal method\n\
    estimates the norm. The internal method will fail if the matrix does not\n\
    have a strong diagonal.\n\
\n\
  use_hmat_filename [string; optional]: Use an old H-matrix possibly to speed up\n\
    making this new one. hmmvp decides whether it can actually use the old file,\n\
    so there is no harm in specifying one. The extension .hm can be omitted.\n\
\n\
  allow_overwrite [[0] or 1]: Allow an H-matrix file to be overwritten by this\n\
    new one?\n\
\n\
  do_extra_tasks_only [[0] or 1]: Execute only what is in DoExtraTasks*().\n\
\n\
  write_Bfro_only [[0] or 1]: Write ||B||_F, either the computed estimate or the\n\
    value passed in field 'Bfro', to the file write_hmat_filename with extension\n\
    .Bfroest. The format is a single number written as %%1.15e\\n. Then exit. If\n\
    this option is not used, no .Bfroest file is written because, in that case,\n\
    it is more accurate to compute ||B||_F from the H-matrix directly.\n\
\n\
  nthreads [[1], >=1]: Number of OpenMP threads, if OpenMP is available and\n\
    enabled.\n\
\n\
  greens_fn [string]: The kernel to use. These are as follows:\n\
\n\
    okada: Wrapper to DC3D [Okada 1992], which is used to compute traction due\n\
           to constant displacement on a rectangle in a homogeneous elastic\n\
           halfspace.\n\
      mu [scalar]: Shear modulus.\n\
      nu [scalar]: Poisson's ratio.\n\
      center [3xNr array]: Rectangle centers in global coordinates. Nr is the\n\
        number of rectangles. -Z is below the surface, so all Z values should\n\
        be <= 0.\n\
      striked [Nr array]: Strike in degrees of a rectangle.\n\
      dipd [Nr array]: Dip in degrees of a rectangle. At 0 strike, the dip\n\
        direction is Y, the strike direction is X, and the element normal is\n\
        +Z. Positive dipd moves the +Y side in the +Z direction. Positive\n\
        striked rotates clockwise about the +Z vector.\n\
      len_along_dip [Nr array]: Length of a rectangle in the dip direction.\n\
      len_along_strike [Nr array]: Length of a rectangle in the strike\n\
        direction.\n\
      disl [3xNr array]: Dislocation vectors in local coordinates. x component\n\
        is strike-slip, y is dip-slip, z is tensile.\n\
      rcv [3xNr array]: Traction vectors in local coordinates.\n\
      src_idxs [array]: Index (starting at 1) set listing source rectangles.\n\
      rcv_idxs [array]: Index set listing receiver rectangles. src_idxs and\n\
        rcv_idxs can overlap. The matrix G is ordered according to src_idxs and\n\
        rcv_idxs: the columns are the sources in the order of src_idxs, and the\n\
        rows are the receivers in the order of rcv_idxs.\n\
      want_fullspace [scalar, optional, default 0]: Non-0 if a fullspace, rather\n\
        than a halfspace, is desired.\n\
\n\
    tgf: Wrapper to tgf [Gibmutas et al], which is used to compute traction due\n\
         to constant displacement on a triangle in a homogeneous elastic\n\
         fullspace or halfspace.\n\
      vertices [3xNv array]: Triangle vertices. Nv is the number of vertices.\n\
      triangles [3xNt array]: Index (starting at 1) into vertices describing the\n\
        triangles.\n\
      mu [scalar]: Shear modulus.\n\
      nu [scalar]: Poisson's ratio.\n\
      disl [3-vector]: Dislocation vector in local coordinates. x component\n\
        is strike-slip, y is dip-slip, z is tensile.\n\
      rcv [3-vector]: Receiver traction vector in local coordinates.\n\
      want_fullspace [scalar, optional, default 0]: Non-0 if a fullspace, rather\n\
        than a halfspace, is desired.\n\
\n\
    btrmbem: 1D line fault in a 2D homogeneous elastic halfspace (plane\n\
    strain). The mesh must be generated by recursively halving segments. I call\n\
    this mesh a tree mesh.\n\
      The geometry is as follows: x = 0 is at the surface. The mesh can start at\n\
    x0 >= 0. If x0 = 0, the boundary condition is the free surface BC. If x0 >\n\
    0, then the BC is that the fault has 0 slip (locked in the usual\n\
    interpretation) in [0, x0]. The right side of the fault is at\n\
        x_max = x0 + sum(nc)*unit_dx.\n\
    The final column of G is for a constant-slip source extending from x_max to\n\
    x = inf. This can be used to implement a velocity BC for x_max.\n\
      The depth of the point x is  x sind(dipdeg).\n\
    Parameters: \n\
      mu [scalar]: Shear modulus.\n\
      nu [scalar]: Poisson's ratio.\n\
      dipdeg [scalar]: Dip angle in degrees of the fault.\n\
      x0 [scalar]: The updip point of the fault.\n\
      unit_dx [scalar]: The size of the smallest element.\n\
      nc [integer array]: Size of an element in terms of unit_dx. nc[i] must be\n\
        a power of 2, and nc as a whole must describe a tree mesh.\n\
      component ['s' or 'n']: Shear or normal.\n\
\n\
    inverse-r: A simple kernel for testing and demonstration.\n\
      X [3xN real array]:\n\
      order [real scalar]:\n\
      delta [real scalar]:\n\
\n\
\n\
"

#endif
